@@ -2,13 +2,14 @@ from copy import copy
 from datetime import datetime, timedelta
 from itertools import groupby
 from typing import Optional
-from urllib.parse import quote as urlquote
+from urllib.parse import parse_qs, urlsplit, quote as urlquote
 
 # import requests  # we use lazy imports to improve load time for local commands
 # from bs4 import BeautifulSoup
 # from bs4.element import NavigableString
 from tqdm import tqdm
 
+from kks.errors import APIError
 from kks.util.h2t import HTML2Text
 
 
@@ -58,6 +59,10 @@ class BaseProblem:
     def contest(self):
         return extract_contest_name(self.short_name)
 
+    def extract_id(self):
+        # TODO store id and generate URL dynamically?
+        return parse_qs(urlsplit(self.href).query)['prob_id'][0]
+
 
 class SummaryProblem(BaseProblem):
     def __init__(self, short_name, name, href, status, tests_passed, score):
@@ -91,16 +96,17 @@ class ProblemWithDeadline:
         self._problem = problem
         self._contest = contest
 
-    def deadline_is_close(self):
-        return self._contest.deadlines.is_close()
-
     def deadline_color(self):
         return self._contest.deadline_color()
 
     def deadline_string(self):
-        return self._contest.deadlines.format_soft()
+        if self._contest.past_deadline():
+            return 'Past deadline'
+        return self._contest.deadlines.to_str(self._contest.active_deadline())
 
     def __getattr__(self, name):
+        if name in ['deadlines', 'past_deadline', 'deadline_is_close', 'active_deadline']:
+            return getattr(self._contest, name)
         return getattr(self._problem, name)
 
 
@@ -259,19 +265,20 @@ class Deadlines:
         self.soft = soft
         self.hard = hard
 
-    def is_close(self):
+    @staticmethod
+    def is_close(deadline):
         from kks.util.storage import Config
-        if self.soft is None:
+        if deadline is None:
             return False
-        dt = self.soft - datetime.now()
+        dt = deadline - datetime.now()
         return dt < timedelta(days=Config().options.deadline_warning_days)
 
-    def format_soft(self):
-        deadline = self.soft
+    @staticmethod
+    def to_str(deadline):
         if deadline is None:
             return 'No deadline'
         result = deadline.strftime(Deadlines.FORMAT)
-        if self.is_close():
+        if Deadlines.is_close(deadline):
             result += ' (!)'
         return result
 
@@ -285,8 +292,18 @@ class ProblemInfo:
         self.current_penalty = current_penalty
         self.deadlines = deadlines
 
+    def active_deadline(self):
+        if self.current_penalty >= self.full_score:
+            return self.deadlines.hard
+        return self.deadlines.soft or self.deadlines.hard
+
+    def deadline_is_close(self):
+        if self.past_deadline():
+            return False
+        return self.deadlines.is_close(self.active_deadline())
+
     def past_deadline(self):
-        return self.deadlines.hard is not None and datetime.now() > self.deadlines.hard or self.current_penalty >= self.full_score
+        return self.deadlines.hard is not None and datetime.now() > self.deadlines.hard
 
 
 class ContestInfo:
@@ -297,9 +314,9 @@ class ContestInfo:
     def deadline_color(self):
         if self.past_deadline():
             return 'red'
-        if self.deadlines.soft is None:
+        if self.active_deadline() is None:
             return 'green'
-        if self.deadlines.is_close():
+        if self.deadline_is_close():
             return 'bright_yellow'
         return 'yellow'
 
@@ -642,8 +659,6 @@ def get_problem_info(problem, cache, session):
 
     # NOTE penalties are assumed to be the same for all tasks in a contest, it may be wrong
 
-    need_loading = False
-
     full_scores = cache.get('full', {})
     run_penalties = cache.get('run', {})
 
@@ -670,7 +685,6 @@ def get_problem_info(problem, cache, session):
 
         if result.past_deadline():
             expiration = None
-            # max_score = 0, will not change
         else:
             if deadlines.soft is not None:
                 expiration = deadlines.soft
@@ -692,18 +706,18 @@ def get_problem_info(problem, cache, session):
     run_penalty = 0  # may be incorrect for kr
     current_penalty = 0
     deadlines = Deadlines(None, None)
+    full_score_found = False
 
     if problem_info is None:
         # if this ever happens, we assume the problem is past the deadline
         deadlines.hard = datetime.fromtimestamp(0)
         return update_cache(full_score, run_penalty, current_penalty, deadlines)
 
-    # TODO "Full score" can be missing (sm01-3)
-    # in this case we should use max score from the table as full
     for row in problem_info.find_all('tr'):
         key, value = row.find_all('td')
         if key.text == 'Full score:':
             full_score = int(value.text)
+            full_score_found = True
         elif key.text == 'Run penalty:':
             run_penalty = int(value.text)
         elif key.text == 'Current penalty:':
@@ -713,6 +727,17 @@ def get_problem_info(problem, cache, session):
         elif key.text == 'Deadline:':
             deadlines.hard = datetime.strptime(value.text, '%Y/%m/%d %H:%M:%S')
 
+    # TODO check API status for running / testing kr contests
+    if not full_score_found:
+        try:
+            problem_status = session.api().problem_status(problem.extract_id()).get('problem', {})
+        except APIError as e:
+            click.secho(f'Cannot get problem info ({problem.short_name}): {e}', err=True)
+            problem_status = {}
+        full_score = problem_status.get('full_score', 0)  # NOTE may be incorrect for kr (?)
+        run_penalty = problem_status.get('run_penalty', 0)
+        # API never tells the deadlines and current penalty
+
     return update_cache(full_score, run_penalty, current_penalty, deadlines)
 
 
